; exploding.scm 
; creates exploding logos
; (c) 1998 Stefan Stiasny
;
; The GIMP -- an image manipulation program
; Copyright (C) 1995 Spencer Kimball and Peter Mattis
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


; ------------
;| Change Log |
; ------------

;; Rel. 0.1 11/08/1998 Initial release by Stefan Stiasny
;; Rel. 0.2 15/12/2020 MrQ: fix up for Gimp 2.10, add RGB to HSV functions, text adjust


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Start Define RGB to HSV functions by GnuTux Glass Effect Text V1.4. ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (convert-rgb-to-hsv color)
	(let* (
			(r (car color))
			(g (cadr color))
			(b (caddr color))
			(cmin (min r (min g b)))
			(cmax (max r (max g b)))
			(diff 0.0)
			(rc 0.0)
			(gc 0.0)
			(bc 0.0)
			(h 0.0)
		  )
			
		(set! diff (- cmax cmin))
		(if (= diff 0.0)
			(set! diff (+ diff 1.0)))
		(set! rc (/ (- cmax r) diff))
		(set! gc (/ (- cmax g) diff))
		(set! bc (/ (- cmax b) diff))
		(set! h  (/ (if (= r cmax)
							(- bc gc)
							(if (= g cmax)
								(+ 2.0 (- rc bc))
								(+ 4.0 (- gc rc))))
						6.0))
		
		(list (if (= cmin cmax)
				0
				(* 360 (if (< h 0.0)
						(+ h 1.0)
						h
					)
				)
			)
			(if (= cmin cmax)
				0
				(/ (- cmax cmin) cmax)
			)
			cmax
		)
	)
)

; RGB to HSV in gimp ranges
(define (rgb-to-hsv color)
	(let*
		(
			(r (car color))
			(g (cadr color))
			(b (caddr color))
			(hsv (convert-rgb-to-hsv (list (/ r 255.0) (/ g 255.0) (/ b 255.0))))
			(h (car hsv))
			(s (cadr hsv))
			(v (caddr hsv))
		)
		(list h (* s 100.0) (* v 100.0))
	)
)

;; End Define RGB to HSV functions by GnuTux
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(define (script-fu-exploding-logo-col text size font color_f letter-spacing line-spacing blur displace_loop color)

(let* (
    (img (car (gimp-image-new size size RGB)))
    (theBgLayer (car (gimp-layer-new img size size RGB "Background" 100 LAYER-MODE-NORMAL-LEGACY)))
    (oldfg (car (gimp-context-get-foreground)))
    (oldbg (car (gimp-context-get-background)))
	(offx 0)
    (offy 0)
    (theTextLayer 0)
    (theText 0)
    (theImageWidth 0)
    (theImageHeight 0)
    (theImageSize 0)
    (theBuffer 0)
    (thePolarLayer 0)
    (theNoiseLayer 0)
    (theGreyLayer 0)
    (theMergedLayer 0)
	(h 0)     ;add MrQ
	(s 0)     ;add MrQ
	(v 0)     ;add MrQ
	(hsv 0)   ;add MrQ
    (i))

 
   ; (gimp-image-undo-disable img)
    (gimp-image-add-layer img theBgLayer 0)
    (gimp-context-set-background '(255 255 255))
    (gimp-context-set-foreground '(0 0 0))
    (gimp-edit-fill theBgLayer 0)
	(gimp-drawable-edit-clear theBgLayer)
    (set! theTextLayer (car (gimp-layer-copy theBgLayer 0)))
    (gimp-drawable-set-name theTextLayer "Text")

    (gimp-image-add-layer img theTextLayer 0)
;                      (gimp-text image drawable x y text border antialias size size-type foundry family weight slant set-width spacing registry encoding)
    (set! theText (car (gimp-text img theTextLayer size size text (* size 2) TRUE size  1 "*" font "medium" "r" "*" "*" "*" "*")))
		(gimp-text-layer-set-base-direction theText 0)	
		(gimp-text-layer-set-justification theText 2)               ; Text Jusification 

		(gimp-text-layer-set-letter-spacing theText letter-spacing)       ; Set Letter Spacing
		(gimp-text-layer-set-line-spacing theText line-spacing)           ; Set Line Spacing	
	
	
    (set! theImageWidth (car (gimp-drawable-width theText)))
    (set! theImageHeight (car (gimp-drawable-height theText)))
   (if (> theImageWidth theImageHeight) (set! theImageSize theImageWidth))           
   (if (> theImageHeight theImageWidth) (set! theImageSize theImageHeight))

    (gimp-image-resize img theImageSize theImageSize 0 0) 
    (gimp-layer-resize theBgLayer theImageSize theImageSize 0 0) 
    (gimp-layer-resize theTextLayer theImageSize theImageSize 0 0) 
	(gimp-edit-fill theBgLayer 0)
	
    ;(set! theBuffer (/ (- theImageSize (car(gimp-drawable-height theText))) 2 ))
    ;(gimp-layer-set-offsets theText 50 theBuffer)
    ;(gimp-floating-sel-anchor theText)
	
	(set! offx (/ (- theImageSize theImageWidth ) 2))
    (set! offy (/ (- theImageSize theImageHeight ) 2))    
    (gimp-layer-set-offsets theText offx offy)
	(gimp-floating-sel-anchor theText)
	
    (plug-in-gauss-iir 1 img theTextLayer blur TRUE TRUE)

    (define (set-pt a index x y)
      (prog1
       (aset a (* index 2) x)
       (aset a (+ (* index 2) 1) y)))

    (define (spline-exploding)
      (let* ((a (cons-array 6 'byte)))
        (set-pt a 0 0 0)
        (set-pt a 1 127 255)
        (set-pt a 2 255 0)
        a))

    (gimp-curves-spline theTextLayer 0 6 (spline-exploding))
    (gimp-curves-spline theTextLayer 0 6 (spline-exploding))

    (set! thePolarLayer (car (gimp-layer-copy theTextLayer 0)))
    (gimp-drawable-set-name thePolarLayer "Polar")
    (gimp-image-add-layer img thePolarLayer 1)
    (plug-in-polar-coords 1 img thePolarLayer 100.000 0.000 FALSE TRUE FALSE)

    (set! theNoiseLayer (car (gimp-layer-new img theImageSize 10 RGB "Noise" 100 LAYER-MODE-LIGHTEN-ONLY-LEGACY)))
    (set! theGreyLayer (car (gimp-layer-new img theImageSize theImageSize RGB "Grey" 100 LAYER-MODE-NORMAL-LEGACY)))

    (gimp-image-add-layer img theNoiseLayer 2)
    (gimp-image-add-layer img theGreyLayer 3)
    (gimp-context-set-background '(128 128 128))
    (gimp-context-set-foreground '(0 0 0))
    (gimp-edit-fill theGreyLayer 1)
    (gimp-edit-fill theNoiseLayer 1)
    (gimp-edit-fill theBgLayer 0)

    (plug-in-noisify 1 img theNoiseLayer 1 0.20 0.20 0.20 0.00)
    (gimp-desaturate theNoiseLayer)
    (gimp-layer-scale theNoiseLayer theImageSize theImageSize 1)
    (gimp-layer-set-offsets theNoiseLayer 0 0)

    (gimp-drawable-set-visible theBgLayer 0)
    (gimp-drawable-set-visible theTextLayer 0)
    (gimp-drawable-set-visible thePolarLayer 0)
    (set! theMergedLayer (car (gimp-image-merge-visible-layers img 2)))

    (plug-in-displace 1 img thePolarLayer 0.000 -100.000 FALSE TRUE theMergedLayer theMergedLayer 0)
  ;;(plug-in-blur 1 img thePolarLayer)
    (plug-in-gauss-iir 1 img thePolarLayer blur TRUE TRUE)

    (set! i displace_loop)
    (while (> i 0)
        (plug-in-displace 1 img thePolarLayer 0.000 -100.000 FALSE TRUE theMergedLayer theMergedLayer 0)
        ;(if (= TRUE blur_toggle)
        ;(plug-in-gauss-iir 1 img thePolarLayer blur TRUE TRUE))
        (set! i (- i 1)))
    
    (plug-in-polar-coords 1 img thePolarLayer 100.000 0.000 FALSE TRUE TRUE)
    (gimp-edit-bucket-fill thePolarLayer BUCKET-FILL-FG LAYER-MODE-NORMAL-LEGACY 100 1 0 1 1)
    ;(gimp-colorize thePolarLayer hue 100 0) <- Deprecated use -> gimp-drawable-colorize-hsl
   (begin   ; add for convert RGB to hsv	
		(set! hsv (rgb-to-hsv color_f))
		(set! h (car hsv))
		(set! s (cadr hsv))
		(set! v (caddr hsv))
		(set! v (- v 100))
		(gimp-drawable-colorize-hsl theTextLayer h s v))	
	
   (begin   ; add for convert RGB to hsv	
		(set! hsv (rgb-to-hsv color))
		(set! h (car hsv))
		(set! s (cadr hsv))
		(set! v (caddr hsv))
		(set! v (- v 100))
		(gimp-drawable-colorize-hsl thePolarLayer h s v))

    (gimp-drawable-set-visible theBgLayer 1)
    (gimp-drawable-set-visible theTextLayer 1)
    (gimp-drawable-set-visible thePolarLayer 1)
    (gimp-drawable-set-visible theMergedLayer 0)
    (gimp-layer-set-mode theTextLayer  LAYER-MODE-HSV-HUE-LEGACY) ; ??? LAYER-MODE-SCREEN-LEGACY (4)  LAYER-MODE-DIFFERENCE-LEGACY (6) LAYER-MODE-OVERLAY-LEGACY


    (gimp-context-set-foreground oldfg)
    (gimp-context-set-background oldbg)
    (gimp-display-new img)
    ;(gimp-image-undo-enable img)
	))

(script-fu-register "script-fu-exploding-logo-col"
                    "Exploding..."
                    "Creates exploding logo"
                    "S.Stiasny/MrQ"
                    "GPL"
                    "1998/2020"
                    ""
                    SF-TEXT         _"Text"    "gimpscripts.net"
                    SF-ADJUSTMENT   _"Font size (pixels)" '(200 2 1000 1 10 0 1)
                    SF-FONT         _"Font"        "Courier"
					SF-COLOR	    _"Font Color HSL"		'(15 40 80)
					SF-ADJUSTMENT   _"Letter Spacing" '(0 -100 100 1 5 0 0)      
					SF-ADJUSTMENT   _"Line Spacing" '(0 -200 200 1 5 0 0)         				
                    SF-ADJUSTMENT  _"Blur Value"  '(5 1 100 1 10 0 1)
                    SF-ADJUSTMENT  _"Explosion Range"  '(10 1 100 1 10 0 1)
					SF-COLOR	   _"Color Explosion HSL"		'(255 155 30))


(script-fu-menu-register "script-fu-exploding-logo-col"
                    _"<Image>/File/Create/Logos")

